/*
This is sample code for NU DC2011 Milestone 2

This code is designed to run with the plotting tool
NU32v2_plot.exe posted on 
http://hades.mech.northwestern.edu/index.php/NU32v2:_Serial_Communication_with_the_PC#Processing

Follow the circuit on http://hades.mech.northwestern.edu/images/e/ea/DC2011_WS2_OpticalIsolation.pdf
and blah.pdf

Mount the phototransistor on the servo with the laser.

Put a pot on B0
Put the h bridge enable on D5, direction on D6, and pwm on D0
Put the servo on D1

By sending this code the letter r, the pot will be read and the value will be written to the Nokia LCD. The motor speed 
will be set to the pot value.

By sending this code the letter d, direction of the motor will be reversed.

By sending this code the letter s, the servo will be rotated for 30 to 120 degrees, the phototransistor read
at each step, and the reads sent back to the plotting program. Point the laser at a cake and see if you can 
tell where it is by the plotted data.
*/


// libraries
#include <plib.h>
#include "NU32v2.h"
#include "Nokia5110.h"

// global variables
int HBridgeDuty;
int ServoDuty;
int adc_buffer[1200];

int main(void) {
  // set PIC32 to max computing power
  SYSTEMConfig(SYS_FREQ, SYS_CFG_ALL);

  // configure for multi-vectored mode
  INTConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);

  // enable multi-vector interrupts 
  INTEnableSystemMultiVectoredInt(); 

  // initialize the serial port and its rx interrupt
  initSerialNU32v2();

  // set up the LED pins (G12 and G13) as outputs
  // and the SW pin (G6) as input
  initLEDs(); 	
  LED0 = 1; // pin is high, but as it is wired the LED is off
  LED1 = 0; // pin is low, but as it is wired the LED is on

  // initialize and clear the Nokia LCD
  LcdInitialize();
  LcdClear();

  // write to the Nokia LCD
  gotoXY(0,0);
  LcdString("DC2011 WS 2");

  // configure and enable the ADC
  CloseADC10();	// ensure the ADC is off before setting the configuration

  // define setup parameters for OpenADC10
  // Turn module on | output in integer | trigger mode auto | enable  autosample
  #define PARAM1  ADC_MODULE_ON | ADC_FORMAT_INTG | ADC_CLK_AUTO | ADC_AUTO_SAMPLING_ON

  // define setup parameters for OpenADC10
  // ADC ref external    | disable offset test    | enable scan mode | perform 2 samples | use one buffer | use MUXA mode
  #define PARAM2  ADC_VREF_AVDD_AVSS | ADC_OFFSET_CAL_DISABLE | ADC_SCAN_ON | ADC_SAMPLES_PER_INT_2 | ADC_ALT_BUF_OFF | ADC_ALT_INPUT_OFF

  // define setup parameters for OpenADC10
  // 				  use ADC internal clock | set sample time
  #define PARAM3  ADC_CONV_CLK_INTERNAL_RC | ADC_SAMPLE_TIME_15

  // define setup parameters for OpenADC10
  // set AN0 and AN1 -> pins B0 and B1
  #define PARAM4	ENABLE_AN0_ANA | ENABLE_AN1_ANA

  // define setup parameters for OpenADC10
  // assign channels you don't want to scan (so list all of the pins that are not in PARAM4
  #define PARAM5	SKIP_SCAN_AN2 | SKIP_SCAN_AN3 | SKIP_SCAN_AN4 | SKIP_SCAN_AN5 | SKIP_SCAN_AN6 | SKIP_SCAN_AN7 | SKIP_SCAN_AN8 | SKIP_SCAN_AN9 | SKIP_SCAN_AN10 | SKIP_SCAN_AN11 | SKIP_SCAN_AN12 | SKIP_SCAN_AN13 | SKIP_SCAN_AN14 | SKIP_SCAN_AN15

  // use ground as neg ref
  SetChanADC10( ADC_CH0_NEG_SAMPLEA_NVREF);
  OpenADC10( PARAM1, PARAM2, PARAM3, PARAM4, PARAM5 ); // configure ADC using parameter define above
  
  // Enable the ADC
  EnableADC10(); 

  // wait for the first conversion to complete so there will be valid data in ADC result registers
  while ( ! mAD1GetIntFlag() ) { } 

  // set up the h-brige enable and direction pins (D5 and D6, but you can use any IO pin you want)
  #define EN  LATDbits.LATD5
  #define DIR LATDbits.LATD6
  LATD  |= 0x0000; // set all of the outputs on D to low
  TRISD &= 0xFF9F; // set pins 5 and 6 in D to outputs (0xFF95 = 0b1111111110011111)
  
  // use Timer2 for a 1kHz PWM signal for the motor on the h-bridge
  OpenTimer2(T2_ON | T2_PS_1_64, 1250); // 80000000Hz / 64ps / 1000Hz = 1250 
                                        // 1250 is < 2^16-1, so it is good to use
                                        // this also means that 1250 corresponds to 100% duty cycle
  OpenOC1(OC_ON | OC_TIMER2_SRC | OC_PWM_FAULT_PIN_DISABLE, 0, 0); // pin D0
  HBridgeDuty = 0;
  SetDCOC1PWM(HBridgeDuty); // duty cycle from 0-1250

  // use Timer3 for a 50Hz PWM signal for the RC servo motor
  OpenTimer3(T3_ON | T2_PS_1_64, 25000); // 80000000Hz / 64ps / 50Hz = 25000 
                                         // 25000 is < 2^16-1, so it is good to use
                                         // this also means that 25000 corresponds to 100% duty cycle
  OpenOC2(OC_ON | OC_TIMER3_SRC | OC_PWM_FAULT_PIN_DISABLE, 0, 0); // pin D1
  ServoDuty = 800; // 30 degrees -> (3000 counts / 180 degrees) * 30 degrees + 300 min val = 800
  SetDCOC2PWM(ServoDuty); // duty cycle from 0-25000
                  // the rc servo has a minimum pulse of 0.24ms for 0 deg and a max of 2.6ms for 180 deg
                  // at 1/(80000000/64)s per pulse, this corresponds to a min value of 300 and a max of 3300
                  // and each value has a resolution of (180 degree / 3000 values) = 0.06 degrees

  LED0 = 0;
  LED1 = 1;

  // turn on the h-bridge
  EN = 1;

  while(1) {
    // nothing, use the UART ISR to control the code
    // if an 'r' is recieved, write the value of the potentiometer to the LCD, and the motor speed is set
    // if a 'd' is recieved, the motor direction is reversed (note the speed will not be corrected!)
    // if an 's' is recieved, sweep the servo from 30 to 60 degrees, reading the phototransistor each time, 
    //                        and after collecting all of the values, send it to the PC over serial
  }

  return 0;
} // end main

// UART3 interrupt handler, priority level 2
void __ISR(_UART_3_VECTOR, ipl2) IntUart3Handler(void) {
  // Is this an RX interrupt?
  if(INTGetFlag(INT_SOURCE_UART_RX(UART3))){
    char data = UARTGetDataByte(UART3);

    // do something with the character
    if(data =='r') {
       // read the analog value on B0 (the potentiometer)
      int adcVal = ReadADC10(0);
      
      // make a character array of the value and write it to the LCD
      gotoXY(0,1);
      char str[10];
      sprintf(str, "pot=%4d", adcVal); // %4d will reserve 4 spaces for the integer
      LcdString(str);

      // set the motor speed
      SetDCOC1PWM(adcVal);
    }

    if (data == 'd') {
      DIR = !DIR;
    }

    if (data == 's') {
      int i = 0;
      int delay = 0;
      // move the servo from 800 to 2000 (30 to 120 degrees)
      // read the phototransistor on pin B1 at each step and save it in a buffer
      for (i=0; i<1200; i++) {
        SetDCOC2PWM(i+800);
        for (delay=0; delay<5000; delay++) {} // wait for the rc servo to move
        adc_buffer[i] = ReadADC10(1);
      }

      // move the servo back to 30 degrees
      SetDCOC2PWM(800);

      // send half the data points to the NU32v2_Plot program
      int j = 0;
      for (i=0; i<1200; i=i+2) {
        sprintf(RS232_Out_Buffer, "a %d %d\r\n", j, adc_buffer[j]/2); // an adc read is 0-1023,
                                                                      // the plotting program max is 511
                                                                      // so divide the read by 2
        WriteString(UART3, RS232_Out_Buffer);
        j++;
      }
    }

    // Clear the RX interrupt Flag
    INTClearFlag(INT_SOURCE_UART_RX(UART3)); 
  }

  // We don't care about TX interrupt
  if(INTGetFlag(INT_SOURCE_UART_TX(UART3))) {
    INTClearFlag(INT_SOURCE_UART_TX(UART3));
  }
}
